% function [B,D]=BD_proj_ls(Gamma,Phi,UUU,SSS,k,delay);
%
% Estimates matrices B,D  using the projection approach.
% with orthonormalization
% Gamma  is the estimated observability matrix
%
% k : number of block rows
%
% UUU is the left factor in the svd of the inputs
% SSS is a vector containing the singular values
%
%  delay : is an 1 x m  row vector, delay(i) is set to 1 force  delay from the
%        i-th input; Dd(:,i) = 0.
%    default : delay = zeros(1,m); 



function [B,D]=BD_proj_ls(Gamma,Phi,UUU,SSS,k,delay);



if ((nargin > 6)| (nargin < 5))
   disp('Wrong number of input arguments');
   return 
end;


if nargin == 5 
   delay = zeros(1,m);
end;


% check dimensions of input matrices.

[rg cg] = size(Gamma);
[rph cph] = size(Phi);

[ru cu] = size(UUU);
rr = length(SSS);





if rg ~=rph
   disp('Gamma and Phi must have the same number of rows')
   return
end;


p = round(rg/k);   % numer of outputs
m = round(cph/k);  % number of inputs
n = cg;     % state dimension

if length(delay) ~= m 
   disp('delay must have m entries')
   return
end;

[U S V] = svd(Gamma);
Gperp = U(:,n+1:rg)'; % rows are orthogonal to column span of Gamma
Hbar = Gperp*Phi;   

pp=rg-n; % number of rows of Gperp
ppp = n+p;

% construct the matrix K
K=[];
for i=0:k-2,
   %K(:,i*ppp+1:(i+1)*ppp) = [Gperp(:,(i+1)*p+1:k*p)*Gamma(1:(k-1-i)*p,:) Gperp(:,i*p+1:(i+1)*p)];
   K = [K Gperp(:,(i+1)*p+1:k*p)*Gamma(1:(k-1-i)*p,:) Gperp(:,i*p+1:(i+1)*p)];
   %HH(i*pp+1:(i+1)*pp,:) = Hbar(:,i*1+1:(i+1)*m);   
end;
%K(:,(k-1)*ppp+1:k*ppp) = [zeros(pp,n) Gperp(:,(k-1)*p+1:k*p)];
K= [K zeros(pp,n) Gperp(:,(k-1)*p+1:k*p)];

%HH((k-1)*pp+1:k*pp,:) = Hbar(:,(k-1)*m+1:k*m);


rr=length(SSS);
[rk ck] = size(K);

KK=zeros(rk*rr,(p+n)*m);

SSS = diag(SSS);
 for jj = 1:rk,
   
   for kk=1:k,
    KK((jj-1)*rr+1:jj*rr,:)=KK((jj-1)*rr+1:jj*rr,:)+kron(UUU(m*(kk-1)+1:kk*m,:)',K(jj,(p+n)*(kk-1)+1:(p+n)*kk));
   end;
   KK((jj-1)*rr+1:jj*rr,:)=SSS*KK((jj-1)*rr+1:jj*rr,:);
 end;
 
 
 %K0 = Hbar;
 %d=n+p;
 %for j = 1:k*m,
 %ZZ(d*(j-1)+1:d*j,:) = K0(:,j);
 %end;
 Hbar =Hbar';
 ZZ = Hbar(:);
 
 
 
 %if delay
 %   KK0=[];
 %   for t=1:m,
 %      KK0 = [KK0 KK(:,(t-1)*(n+p)+1:(t-1)*(n+p)+n)];
 %   end;
 %   KK =KK0;
 %  
 %   BD = KK\ZZ;
 %   for t=1:m,
 %    B(:,t)=BD((t-1)*n+1:t*n);
 %   end;  
 %   D = zeros(p,m);
 %else
 %   BD = KK\ZZ;
 %   for t=1:m,
 %    B(:,t)=BD((t-1)*(n+p)+1:(t-1)*(n+p)+n);
 %    D(:,t)=BD((t-1)*(n+p)+n+1:t*(n+p));
 %   end;   
 %end;
 
 %if delay
 %   KK = KK(:,1:m*n);
 %   BD = KK\ZZ;
 %   for t=1:m,
 %    B(:,t)=BD((t-1)*n+1:t*n);
 %  end;  
 %   D = zeros(p,m);
 %else
 %   BD = KK\ZZ;
 %   for t=1:m,
 %    B(:,t)=BD((t-1)*n+1:t*n);
 %    D(:,t)=BD(m*n+(t-1)*p+1:m*n+t*p);
 %   end;   
 %end;

  I = find(delay); % find inputs which have to be delayed
 
 if ~isempty(I)
    KK0=[];       
    prev=0;
    for i=1:length(I),
       t=I(i);
       KK0 = [KK0 KK(:,prev*(n+p)+1:(t-1)*(n+p)+n)];
       prev = t;
    end;
    if prev < m
     KK0 = [KK0 KK(:,prev*(n+p)+1:m*(n+p))];
    end
 
   
    KK =KK0;
   
    BD = KK\ZZ;
    prev=0;
    D=zeros(p,m);
    for i=1:length(I),
       t=I(i);
       for j = prev+1:t-1,
         B(:,j)=BD((j-1)*n+(j-i)*p+1:(j-1)*n+(j-i)*p+n);
         D(:,j)=BD(j*n+(j-i)*p+1:j*n+(j-i+1)*p);
       end;
       B(:,t)=BD((t-1)*n+(t-i)*p+1:(t-1)*n+(t-i)*p+n);
       prev = t;
    end;
    if prev <m 
      for j = prev+1:m,
           B(:,j)=BD((j-1)*n+(j-i-1)*p+1:(j-1)*n+(j-i-1)*p+n);
           D(:,j)=BD(j*n+(j-i-1)*p+1:j*n+(j-i)*p); 
      end;
   end;
   

   % for t=1:m,
   %  B(:,t)=BD((t-1)*n+1:t*n);
   % end;  
   % D = zeros(p,m);
 else
    BD = KK\ZZ;
    for t=1:m,
     B(:,t)=BD((t-1)*(n+p)+1:(t-1)*(n+p)+n);
     D(:,t)=BD((t-1)*(n+p)+n+1:t*(n+p));
    end;   
 end;
 


 
 
 
 
 
 












