% function [As,Ks,Cs,Lambda]=stoc_est_cva(y,ks,ns);
%
%
% (As,Ks,Cs,I) is an innovation representation of the stochastic component
%  Lambda is the variance of the innovation
%
%  y : output data as row vector y = [y(1), y(2),......,y(Ndat)]
%  ks dimension of past and future in stochastic identification
%  ns  choosen order
%

function [As,Ks,Cs,Lambda]=stoc_est_cva(y,ks,ns);

if ((nargin > 3) | (nargin < 2)) 
   disp('Wrong number of input arguments')
   return
end;


if ((nargin < 3) | isempty(ns))
   ns=[1:10];
end;


if ((nargin < 2) | isempty(ks))
   ksp = ceil(1.2*max(ns)+3); % as in the N4SID file of matlab
   ksf = ksp;
else
   ksp = ks(1);
   ksf = ks(2);
end;

[p,Ny]=size(y);


N = Ny-(ksf+ksp)+1;

%%% Hankel matrices:

Y = Hankel(y,ksf+ksp,N);
Yp = Y(1:ksp*p,:);
Yf = Y(ksp*p+1:(ksf+ksp)*p,:);


[Q,R] =qr(Y',0); % economy size qr decomposition 
Q=Q(:,1:(ksf+ksp)*p)';
R=triu(R(1:(ksf+ksp)*p,1:(ksf+ksp)*p))';


Yup = R(ksp*p+1:(ksf+ksp)*p-p,1:ksp*p);
Yup1 =R(ksp*p+p+1:(ksf+ksp)*p,1:ksp*p+p);

Lf = chol(Yf(1:(ksf-1)*p,:)*Yf(1:(ksf-1)*p,:)');
[U S V] = svd(pinv(Lf')*R(ksp*p+1:(ksf+ksp)*p-p,1:ksp*p)); % cva



if min(ns)<max(ns),     % if def_order
      figure
      dS=diag(S);
      testo=(diag(S));
      ndef=max(max(find(testo>(max(testo)+min(testo))/2)));
      bar(((testo)));
      disp('Press any key to continue');
       
      title('Select model order in command window.')
      disp(['default=', num2str(ndef)])
      n=input('Select model order:(''Return'' gives default) ');
      if isempty(n)
         %testo=log(diag(SS));
         n=ndef;
         disp(['Order chosen to ',int2str(n)]);
      end
      close;
     else n=ns; 
end  
 
S1=S(1:n,1:n);
Gammak= U(:,1:n)*S1^(1/2);

pGammak=pinv(Gammak); 
pGammak = pGammak*pinv(Lf');
  
Zkx = [pGammak*Yup];
Zk1x = [pGammak*Yup1(:,1:ksp*p)];

pZkx=pinv(Zkx); 

As = real(Zk1x*pZkx);
Cs = real(R(p*ksp+1:p*ksp+p,1:ksp*p)*pZkx);
Csb = R(p*ksp+1-p:p*ksp,1:ksp*p)*pZkx*S1;
Csb=Csb';
Lam = R(p*ksp+1:p*ksp+p,1:ksp*p+p)*R(p*ksp+1:p*ksp+p,1:ksp*p+p)';


     %% Solve the Riccati equation 

     iLam = inv(Lam);
     [W,d] = eig([ As'-Cs'*iLam*Csb' zeros(n,n);...
          -Csb*iLam*Csb'   eye(n) ], ...
        [eye(n) -Cs'*iLam*Cs; ...
         zeros(n,n) As-Csb*iLam*Cs ]);
     d = diag(d);
     [e,index] = sort(abs(d));        % sort on magnitude of eigenvalues
     if (~((e(n) < 1) & (e(n+1)>1)))
           disp('Can''t order eigenvalues');
     end

    % select vectors with eigenvalues inside unit circle
    WW = W(:,index(1:n));

    %%% State covariance:
    
    P = WW(n+1:2*n,:)*inv(WW(1:n,:));
   
    Q = real(P-As*P*As');
    S = real(Csb-As*P*Cs');
    Lambda = real(Lam-Cs*P*Cs');
    
    Ks = S*pinv(Lambda);